Crate pom [−] [src]
Modules
| char_class |
Utility functions to get char class of byte value. |
| parser |
Constains predefined parsers and combinators. |
Structs
| DataInput |
Wrap &[u8] or &[char] as input to parser. |
| TextInput |
Wrap &str as input to parser. |
Enums
| Error |
Parser error. |
Traits
| Input |
Parser input is generic over terminal type, which is usually u8 or char. |
| Train |
A train is a series of knots, each knot can be a terminal symbol. Train is used as argument of seq(), one_of() and none_of(), so that both b'[u8] literal' and "string literal" can be accepted. |
Type Definitions
| Parser |
Parser type, |
| Result |
Parser result, |